package net.gimite.nativeexe;

import java.io.BufferedReader;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import net.gimite.nativeexe.R;
import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.*;

public class MainActivity extends Activity {
	
    private TextView outputView;
	private Button localRunButton;
	private EditText localPathEdit;
	private Handler handler = new Handler();
	private EditText urlEdit;
	private Button remoteRunButton;

	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        outputView = (TextView)findViewById(R.id.outputView);
        localPathEdit = (EditText)findViewById(R.id.localPathEdit);
        localRunButton = (Button)findViewById(R.id.localRunButton);
        localRunButton.setOnClickListener(onLocalRunButtonClick);
        urlEdit = (EditText)findViewById(R.id.urlEdit);
        remoteRunButton = (Button)findViewById(R.id.remoteRunButton);
        remoteRunButton.setOnClickListener(onRemoteRunButtonClick);
    }
    
	private OnClickListener onLocalRunButtonClick = new OnClickListener() {
		public void onClick(View v) {
			String output = exec(localPathEdit.getText().toString(), null, null);
			output(output);
		}
	};

	private OnClickListener onRemoteRunButtonClick = new OnClickListener() {
		public void onClick(View v) {
			final String url = urlEdit.getText().toString();
			final String localPath = "/data/data/net.gimite.nativeexe/a.out";
			output("Downloading...");
			Thread thread = new Thread(new Runnable() {
				public void run() {
					download(url, localPath);
					exec("/system/bin/chmod", "744", localPath);
					output("Executing...");
					String output = exec(localPath, null, null);
					output(output);
				}
			});
			thread.start();
		}
	};

	// Executes UNIX command.
    private String exec(String arg0, String arg1, String arg2) {
		try {
			// android.os.Exec is not included in android.jar so we need to use reflection.
			Class<?> execClass = Class.forName("android.os.Exec");
	        Method createSubprocess = execClass.getMethod("createSubprocess",
	        		String.class, String.class, String.class, int[].class);
	        Method waitFor = execClass.getMethod("waitFor", int.class);

	        // Executes the command.
	        // NOTE: createSubprocess() is asynchronous.
	        int[] pid = new int[1];
	        FileDescriptor fd = (FileDescriptor)createSubprocess.invoke(
	        		null, arg0, arg1, arg2, pid);
	        
	        // Reads stdout.
	        // NOTE: You can write to stdin of the command using new FileOutputStream(fd).
	        FileInputStream in = new FileInputStream(fd);
	        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
	        String output = "";
	        try {
		        String line;
				while ((line = reader.readLine()) != null) {
					output += line + "\n";
				}
			} catch (IOException e) {
				// It seems IOException is thrown when it reaches EOF.
			}

			// Waits for the command to finish.
			waitFor.invoke(null, pid[0]);
			
			return output;
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e.getMessage());
		} catch (SecurityException e) {
			throw new RuntimeException(e.getMessage());
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e.getMessage());
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e.getMessage());
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e.getMessage());
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e.getMessage());
		}
    }
    
    private void download(String urlStr, String localPath) {
    	try {
			URL url = new URL(urlStr);
			HttpURLConnection urlconn = (HttpURLConnection)url.openConnection();
			urlconn.setRequestMethod("GET");
			urlconn.setInstanceFollowRedirects(true);
			urlconn.connect();
			InputStream in = urlconn.getInputStream();
			FileOutputStream out = new FileOutputStream(localPath);
			int read;
			byte[] buffer = new byte[4096];
			while ((read = in.read(buffer)) > 0) {
				out.write(buffer, 0, read);
			}
			out.close();
			in.close();
			urlconn.disconnect();
		} catch (MalformedURLException e) {
			throw new RuntimeException(e.getMessage());
		} catch (IOException e) {
			throw new RuntimeException(e.getMessage());
		}
    }
    
    private void output(final String str) {
    	Runnable proc = new Runnable() {
			public void run() {
				outputView.setText(str);
			}
    	};
    	handler.post(proc);
    }
    
}